/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Assembly Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <string.h>

#include <oo.h>
#include "OO7.h"
#include "GenParams.h"
#include "VarParams.h"

extern ooTrans	xact;
extern int	nextComplexAssemblyId;
extern int	nextBaseAssemblyId;
extern int	debugMode;
extern char*	types[NumTypes];

extern BAIdList* private_cp;
extern BAIdList* shared_cp;

int    numCompositePartsLinked = 0;

//////////////////////////////////////////////////////////////////////////
// 
// DoNothing method for Assemblies 
// 
//////////////////////////////////////////////////////////////////////////

void Assembly::DoNothing ()
{
    if (id < 0) 
        printf("DoNothing: negative id.\n");
    if (debugMode) {
        printf("==> DoNothing(id = %d, type = %s, buildDate = %d)\n",
                               id, type, buildDate);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// ComplexAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

ComplexAssembly::ComplexAssembly(int32 asId,
    	                         ooHandle(ComplexAssembly)& parentAssembly,
		                 int32 levelNo)
{
    if (debugMode) {
        printf("ComplexAssembly::ComplexAssembly(asId = %d, levelNo = %d)\n",
                asId, levelNo);
    }

    // initialize the simple stuff (some of it randomly)

    id = asId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));

    if (!parentAssembly.isNull()) { set_superAssembly(parentAssembly); }

    // now recursively create subassemblies for this complex assembly

    int i, nextId;
    ooHandle(ComplexAssembly) assmH = ooThis();

    ooKey key;
    key.type = oocInt32;
    key.offset = ooGetMemberOffset(BaseAssembly, id);
    key.size   = ooGetMemberSize(BaseAssembly, id);
    key.value  = &nextId;

    for (i = 0; i < NumAssmPerAssm; i++) {

        if (levelNo < NumAssmLevels-1) {
            nextId = nextComplexAssemblyId++;
            ooHandle(ComplexAssembly) subAssmH = new(assmH) ComplexAssembly
	        (nextId, assmH, levelNo+1);
        } else {
            nextId = nextBaseAssemblyId++;
            ooHandle(BaseAssembly) subAssmH = ooNewKey(BaseAssembly,
	                 (nextId, assmH), assmH, key);
        }
    }
}


////////////////////////////////////////////////////////////////////////////
//
// BaseAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

BaseAssembly::BaseAssembly(int32 asId,
    	                   ooHandle(ComplexAssembly)& parentAssembly)
{
    if (debugMode) {
        printf("BaseAssembly::BaseAssembly(asId = %d, levelNo = %d)\n",
                asId, NumAssmLevels);
    }

    // initialize the simple stuff (some of it randomly)

    id = asId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));

    if (!parentAssembly.isNull()) { set_superAssembly(parentAssembly); }

    // figure out which module contains this base assembly

    ooHandle(BaseAssembly) thisH = ooThis();
    ooHandle(Module) modH = (ooHandle(Module)&) thisH.containedIn();

    // get access to the design library containing composite parts

    ooHandle(ooDBObj) databaseH = modH.containedIn();
    ooHandle(ooContObj) designLibH;
    designLibH.open(databaseH, "DesignLibrary", oocRead);
    if (modH.isValid() == oocFalse) {
        fprintf(stderr, "ERROR: Unable to access DesignLibrary\n");
	xact.abort();
        exit(1);
    }

    // get ready to select composite parts via their id index
    ooKey key;
    int compId;
    key.type = oocInt32;
    key.offset = ooGetMemberOffset(CompositePart, id);
    key.size   = ooGetMemberSize(CompositePart, id);
    key.value  = &compId;

    // select the private composite parts for this assembly

    int i;
    int lowCompId = (modH->id - 1) * NumCompPerModule + 1;
    int compIdLimit = NumCompPerModule;
    ooHandle(CompositePart) compH;

    if (debugMode) { printf("[Private parts = { "); }

    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = lowCompId + (int) (random() % compIdLimit);
	private_cp[compId].insert(ooThis());  // keep track of which
		// composite parts this base assembly uses as private
        if (debugMode) { printf("%d ", compId); }
    }

    // now select the shared composite parts for this assembly

    if (debugMode) { printf("}, Shared parts = { "); }

    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = (int) (random() % TotalCompParts) + 1;
	shared_cp[compId].insert(ooThis());  // keep track of which
		 // composite parts this base assembly uses as shared
        if (debugMode) { printf("%d ", compId); }
    }
    if (debugMode) { printf("}\n"); }
}
