/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

/* C++ 'references' header file: Objectivity/DB DDL version 2.1.0 */


#ifndef _OO7_REF_H
#define _OO7_REF_H



extern const ooTypeNumber ooTypeN(AtomicPart);
extern const ooAssocNumber ooAssocN(AtomicPart,to);
extern const ooAssocNumber ooAssocN(AtomicPart,from);
extern const ooAssocNumber ooAssocN(AtomicPart,partOf);


class AtomicPart;
class ooRef(AtomicPart);
class ooShortRef(AtomicPart);
class ooHandle(AtomicPart) : public ooHandle(ooObj) {
  public:
    ooHandle(AtomicPart)();
    ooHandle(AtomicPart)(const ooHandle(AtomicPart)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(AtomicPart)(const ooRef(AtomicPart)& id);
    ooHandle(AtomicPart)(const AtomicPart* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(AtomicPart)& operator=(const ooHandle(AtomicPart)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(AtomicPart)& operator=(const ooRef(AtomicPart)& id);
    ooHandle(AtomicPart)& operator=(const ooShortRef(AtomicPart)& shortId);
    ooHandle(AtomicPart)& operator=(const AtomicPart* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(AtomicPart), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(AtomicPart), openMode); }
    ooHandle(AtomicPart)& copy(const ooHandle(ooObj)& nearH, ooHandle(AtomicPart)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(AtomicPart) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(AtomicPart) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    AtomicPart* vm()         { return (AtomicPart*)get_vm(); }
    AtomicPart* operator->() { return (AtomicPart*)get_vm(); }
    AtomicPart& operator*();
    operator AtomicPart*();
};

class ooItr(AtomicPart) : public ooHandle(AtomicPart) {
  public:
    ooItr(AtomicPart)();
    ~ooItr(AtomicPart)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(AtomicPart)(ooItr(AtomicPart)&) { }
    void operator=(ooItr(AtomicPart)&) { }
};

class ooDeref(AtomicPart) : public ooDerefBase {
    friend ooRef(AtomicPart);
    ooDeref(AtomicPart)(ooDeref(AtomicPart)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(AtomicPart)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(AtomicPart)() { unpin(); }
public:
    AtomicPart *operator->() { return (AtomicPart *)vm(); }
};

class ooRef(AtomicPart) : public ooRef(ooObj) {
  public:
    ooRef(AtomicPart)() { }
    ooRef(AtomicPart)(const ooRef(AtomicPart)& id) : ooRef(ooObj)(id) { }
    ooRef(AtomicPart)(const ooHandle(AtomicPart)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(AtomicPart)(const AtomicPart* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(AtomicPart)& operator=(const ooRef(AtomicPart)& id) { ooId::operator=(id); return *this; }
    ooRef(AtomicPart)& operator=(const ooShortRef(AtomicPart)& shortId);
    ooRef(AtomicPart)& operator=(const ooHandle(AtomicPart)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(AtomicPart)& operator=(const AtomicPart* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    AtomicPart* vm(const ooMode openMode = oocRead)
        { return (AtomicPart*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(AtomicPart) operator->() { return (ooDeref(AtomicPart)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(AtomicPart) : public ooShortRef(ooObj) {
  public:
    ooShortRef(AtomicPart)();
    ooShortRef(AtomicPart)(const ooShortRef(AtomicPart)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(AtomicPart)(const ooRef(AtomicPart)& id) { ooShortId::operator=(id); }
    ooShortRef(AtomicPart)(const ooHandle(AtomicPart)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(AtomicPart)(const AtomicPart* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(AtomicPart)& operator=(const ooShortRef(AtomicPart)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(AtomicPart)& operator=(const ooRef(AtomicPart)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(AtomicPart)& operator=(const ooHandle(AtomicPart)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(AtomicPart)& operator=(const AtomicPart* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(AtomicPart)::ooHandle(AtomicPart)(const ooRef(AtomicPart)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(AtomicPart)& ooHandle(AtomicPart)::operator=(const ooRef(AtomicPart)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(AtomicPart)& ooHandle(AtomicPart)::operator=(const ooShortRef(AtomicPart)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(AtomicPart)& ooRef(AtomicPart)::operator=(const ooShortRef(AtomicPart)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(Connection);
extern const ooAssocNumber ooAssocN(Connection,from);
extern const ooAssocNumber ooAssocN(Connection,to);


class Connection;
class ooRef(Connection);
class ooShortRef(Connection);
class ooHandle(Connection) : public ooHandle(ooObj) {
  public:
    ooHandle(Connection)();
    ooHandle(Connection)(const ooHandle(Connection)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(Connection)(const ooRef(Connection)& id);
    ooHandle(Connection)(const Connection* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(Connection)& operator=(const ooHandle(Connection)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(Connection)& operator=(const ooRef(Connection)& id);
    ooHandle(Connection)& operator=(const ooShortRef(Connection)& shortId);
    ooHandle(Connection)& operator=(const Connection* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(Connection), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(Connection), openMode); }
    ooHandle(Connection)& copy(const ooHandle(ooObj)& nearH, ooHandle(Connection)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(Connection) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(Connection) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    Connection* vm()         { return (Connection*)get_vm(); }
    Connection* operator->() { return (Connection*)get_vm(); }
    Connection& operator*();
    operator Connection*();
};

class ooItr(Connection) : public ooHandle(Connection) {
  public:
    ooItr(Connection)();
    ~ooItr(Connection)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(Connection)(ooItr(Connection)&) { }
    void operator=(ooItr(Connection)&) { }
};

class ooDeref(Connection) : public ooDerefBase {
    friend ooRef(Connection);
    ooDeref(Connection)(ooDeref(Connection)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(Connection)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(Connection)() { unpin(); }
public:
    Connection *operator->() { return (Connection *)vm(); }
};

class ooRef(Connection) : public ooRef(ooObj) {
  public:
    ooRef(Connection)() { }
    ooRef(Connection)(const ooRef(Connection)& id) : ooRef(ooObj)(id) { }
    ooRef(Connection)(const ooHandle(Connection)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(Connection)(const Connection* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(Connection)& operator=(const ooRef(Connection)& id) { ooId::operator=(id); return *this; }
    ooRef(Connection)& operator=(const ooShortRef(Connection)& shortId);
    ooRef(Connection)& operator=(const ooHandle(Connection)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(Connection)& operator=(const Connection* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    Connection* vm(const ooMode openMode = oocRead)
        { return (Connection*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(Connection) operator->() { return (ooDeref(Connection)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(Connection) : public ooShortRef(ooObj) {
  public:
    ooShortRef(Connection)();
    ooShortRef(Connection)(const ooShortRef(Connection)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(Connection)(const ooRef(Connection)& id) { ooShortId::operator=(id); }
    ooShortRef(Connection)(const ooHandle(Connection)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(Connection)(const Connection* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(Connection)& operator=(const ooShortRef(Connection)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(Connection)& operator=(const ooRef(Connection)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(Connection)& operator=(const ooHandle(Connection)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(Connection)& operator=(const Connection* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(Connection)::ooHandle(Connection)(const ooRef(Connection)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(Connection)& ooHandle(Connection)::operator=(const ooRef(Connection)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(Connection)& ooHandle(Connection)::operator=(const ooShortRef(Connection)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(Connection)& ooRef(Connection)::operator=(const ooShortRef(Connection)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(CompositePart);
extern const ooAssocNumber ooAssocN(CompositePart,documentation);
extern const ooAssocNumber ooAssocN(CompositePart,usedInPriv);
extern const ooAssocNumber ooAssocN(CompositePart,usedInShar);
extern const ooAssocNumber ooAssocN(CompositePart,parts);
extern const ooAssocNumber ooAssocN(CompositePart,rootPart);


class CompositePart;
class ooRef(CompositePart);
class ooShortRef(CompositePart);
class ooHandle(CompositePart) : public ooHandle(ooObj) {
  public:
    ooHandle(CompositePart)();
    ooHandle(CompositePart)(const ooHandle(CompositePart)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(CompositePart)(const ooRef(CompositePart)& id);
    ooHandle(CompositePart)(const CompositePart* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(CompositePart)& operator=(const ooHandle(CompositePart)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(CompositePart)& operator=(const ooRef(CompositePart)& id);
    ooHandle(CompositePart)& operator=(const ooShortRef(CompositePart)& shortId);
    ooHandle(CompositePart)& operator=(const CompositePart* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(CompositePart), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(CompositePart), openMode); }
    ooHandle(CompositePart)& copy(const ooHandle(ooObj)& nearH, ooHandle(CompositePart)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(CompositePart) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(CompositePart) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    CompositePart* vm()         { return (CompositePart*)get_vm(); }
    CompositePart* operator->() { return (CompositePart*)get_vm(); }
    CompositePart& operator*();
    operator CompositePart*();
};

class ooItr(CompositePart) : public ooHandle(CompositePart) {
  public:
    ooItr(CompositePart)();
    ~ooItr(CompositePart)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(CompositePart)(ooItr(CompositePart)&) { }
    void operator=(ooItr(CompositePart)&) { }
};

class ooDeref(CompositePart) : public ooDerefBase {
    friend ooRef(CompositePart);
    ooDeref(CompositePart)(ooDeref(CompositePart)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(CompositePart)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(CompositePart)() { unpin(); }
public:
    CompositePart *operator->() { return (CompositePart *)vm(); }
};

class ooRef(CompositePart) : public ooRef(ooObj) {
  public:
    ooRef(CompositePart)() { }
    ooRef(CompositePart)(const ooRef(CompositePart)& id) : ooRef(ooObj)(id) { }
    ooRef(CompositePart)(const ooHandle(CompositePart)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(CompositePart)(const CompositePart* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(CompositePart)& operator=(const ooRef(CompositePart)& id) { ooId::operator=(id); return *this; }
    ooRef(CompositePart)& operator=(const ooShortRef(CompositePart)& shortId);
    ooRef(CompositePart)& operator=(const ooHandle(CompositePart)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(CompositePart)& operator=(const CompositePart* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    CompositePart* vm(const ooMode openMode = oocRead)
        { return (CompositePart*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(CompositePart) operator->() { return (ooDeref(CompositePart)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(CompositePart) : public ooShortRef(ooObj) {
  public:
    ooShortRef(CompositePart)();
    ooShortRef(CompositePart)(const ooShortRef(CompositePart)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(CompositePart)(const ooRef(CompositePart)& id) { ooShortId::operator=(id); }
    ooShortRef(CompositePart)(const ooHandle(CompositePart)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(CompositePart)(const CompositePart* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(CompositePart)& operator=(const ooShortRef(CompositePart)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(CompositePart)& operator=(const ooRef(CompositePart)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(CompositePart)& operator=(const ooHandle(CompositePart)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(CompositePart)& operator=(const CompositePart* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(CompositePart)::ooHandle(CompositePart)(const ooRef(CompositePart)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(CompositePart)& ooHandle(CompositePart)::operator=(const ooRef(CompositePart)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(CompositePart)& ooHandle(CompositePart)::operator=(const ooShortRef(CompositePart)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(CompositePart)& ooRef(CompositePart)::operator=(const ooShortRef(CompositePart)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(Document);
extern const ooAssocNumber ooAssocN(Document,part);


class Document;
class ooRef(Document);
class ooShortRef(Document);
class ooHandle(Document) : public ooHandle(ooObj) {
  public:
    ooHandle(Document)();
    ooHandle(Document)(const ooHandle(Document)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(Document)(const ooRef(Document)& id);
    ooHandle(Document)(const Document* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(Document)& operator=(const ooHandle(Document)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(Document)& operator=(const ooRef(Document)& id);
    ooHandle(Document)& operator=(const ooShortRef(Document)& shortId);
    ooHandle(Document)& operator=(const Document* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(Document), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(Document), openMode); }
    ooHandle(Document)& copy(const ooHandle(ooObj)& nearH, ooHandle(Document)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(Document) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(Document) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    Document* vm()         { return (Document*)get_vm(); }
    Document* operator->() { return (Document*)get_vm(); }
    Document& operator*();
    operator Document*();
};

class ooItr(Document) : public ooHandle(Document) {
  public:
    ooItr(Document)();
    ~ooItr(Document)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(Document)(ooItr(Document)&) { }
    void operator=(ooItr(Document)&) { }
};

class ooDeref(Document) : public ooDerefBase {
    friend ooRef(Document);
    ooDeref(Document)(ooDeref(Document)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(Document)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(Document)() { unpin(); }
public:
    Document *operator->() { return (Document *)vm(); }
};

class ooRef(Document) : public ooRef(ooObj) {
  public:
    ooRef(Document)() { }
    ooRef(Document)(const ooRef(Document)& id) : ooRef(ooObj)(id) { }
    ooRef(Document)(const ooHandle(Document)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(Document)(const Document* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(Document)& operator=(const ooRef(Document)& id) { ooId::operator=(id); return *this; }
    ooRef(Document)& operator=(const ooShortRef(Document)& shortId);
    ooRef(Document)& operator=(const ooHandle(Document)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(Document)& operator=(const Document* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    Document* vm(const ooMode openMode = oocRead)
        { return (Document*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(Document) operator->() { return (ooDeref(Document)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(Document) : public ooShortRef(ooObj) {
  public:
    ooShortRef(Document)();
    ooShortRef(Document)(const ooShortRef(Document)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(Document)(const ooRef(Document)& id) { ooShortId::operator=(id); }
    ooShortRef(Document)(const ooHandle(Document)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(Document)(const Document* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(Document)& operator=(const ooShortRef(Document)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(Document)& operator=(const ooRef(Document)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(Document)& operator=(const ooHandle(Document)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(Document)& operator=(const Document* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(Document)::ooHandle(Document)(const ooRef(Document)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(Document)& ooHandle(Document)::operator=(const ooRef(Document)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(Document)& ooHandle(Document)::operator=(const ooShortRef(Document)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(Document)& ooRef(Document)::operator=(const ooShortRef(Document)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(Manual);
extern const ooAssocNumber ooAssocN(Manual,mod);


class Manual;
class ooRef(Manual);
class ooShortRef(Manual);
class ooHandle(Manual) : public ooHandle(ooObj) {
  public:
    ooHandle(Manual)();
    ooHandle(Manual)(const ooHandle(Manual)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(Manual)(const ooRef(Manual)& id);
    ooHandle(Manual)(const Manual* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(Manual)& operator=(const ooHandle(Manual)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(Manual)& operator=(const ooRef(Manual)& id);
    ooHandle(Manual)& operator=(const ooShortRef(Manual)& shortId);
    ooHandle(Manual)& operator=(const Manual* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(Manual), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(Manual), openMode); }
    ooHandle(Manual)& copy(const ooHandle(ooObj)& nearH, ooHandle(Manual)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(Manual) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(Manual) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    Manual* vm()         { return (Manual*)get_vm(); }
    Manual* operator->() { return (Manual*)get_vm(); }
    Manual& operator*();
    operator Manual*();
};

class ooItr(Manual) : public ooHandle(Manual) {
  public:
    ooItr(Manual)();
    ~ooItr(Manual)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(Manual)(ooItr(Manual)&) { }
    void operator=(ooItr(Manual)&) { }
};

class ooDeref(Manual) : public ooDerefBase {
    friend ooRef(Manual);
    ooDeref(Manual)(ooDeref(Manual)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(Manual)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(Manual)() { unpin(); }
public:
    Manual *operator->() { return (Manual *)vm(); }
};

class ooRef(Manual) : public ooRef(ooObj) {
  public:
    ooRef(Manual)() { }
    ooRef(Manual)(const ooRef(Manual)& id) : ooRef(ooObj)(id) { }
    ooRef(Manual)(const ooHandle(Manual)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(Manual)(const Manual* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(Manual)& operator=(const ooRef(Manual)& id) { ooId::operator=(id); return *this; }
    ooRef(Manual)& operator=(const ooShortRef(Manual)& shortId);
    ooRef(Manual)& operator=(const ooHandle(Manual)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(Manual)& operator=(const Manual* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    Manual* vm(const ooMode openMode = oocRead)
        { return (Manual*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(Manual) operator->() { return (ooDeref(Manual)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(Manual) : public ooShortRef(ooObj) {
  public:
    ooShortRef(Manual)();
    ooShortRef(Manual)(const ooShortRef(Manual)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(Manual)(const ooRef(Manual)& id) { ooShortId::operator=(id); }
    ooShortRef(Manual)(const ooHandle(Manual)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(Manual)(const Manual* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(Manual)& operator=(const ooShortRef(Manual)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(Manual)& operator=(const ooRef(Manual)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(Manual)& operator=(const ooHandle(Manual)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(Manual)& operator=(const Manual* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(Manual)::ooHandle(Manual)(const ooRef(Manual)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(Manual)& ooHandle(Manual)::operator=(const ooRef(Manual)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(Manual)& ooHandle(Manual)::operator=(const ooShortRef(Manual)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(Manual)& ooRef(Manual)::operator=(const ooShortRef(Manual)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(Assembly);
extern const ooAssocNumber ooAssocN(Assembly,superAssembly);


class Assembly;
class ooRef(Assembly);
class ooShortRef(Assembly);
class ooHandle(Assembly) : public ooHandle(ooObj) {
  public:
    ooHandle(Assembly)();
    ooHandle(Assembly)(const ooHandle(Assembly)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(Assembly)(const ooRef(Assembly)& id);
    ooHandle(Assembly)(const Assembly* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(Assembly)& operator=(const ooHandle(Assembly)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(Assembly)& operator=(const ooRef(Assembly)& id);
    ooHandle(Assembly)& operator=(const ooShortRef(Assembly)& shortId);
    ooHandle(Assembly)& operator=(const Assembly* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(Assembly), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(Assembly), openMode); }
    ooHandle(Assembly)& copy(const ooHandle(ooObj)& nearH, ooHandle(Assembly)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(Assembly) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(Assembly) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    Assembly* vm()         { return (Assembly*)get_vm(); }
    Assembly* operator->() { return (Assembly*)get_vm(); }
    Assembly& operator*();
    operator Assembly*();
};

class ooItr(Assembly) : public ooHandle(Assembly) {
  public:
    ooItr(Assembly)();
    ~ooItr(Assembly)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(Assembly)(ooItr(Assembly)&) { }
    void operator=(ooItr(Assembly)&) { }
};

class ooDeref(Assembly) : public ooDerefBase {
    friend ooRef(Assembly);
    ooDeref(Assembly)(ooDeref(Assembly)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(Assembly)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(Assembly)() { unpin(); }
public:
    Assembly *operator->() { return (Assembly *)vm(); }
};

class ooRef(Assembly) : public ooRef(ooObj) {
  public:
    ooRef(Assembly)() { }
    ooRef(Assembly)(const ooRef(Assembly)& id) : ooRef(ooObj)(id) { }
    ooRef(Assembly)(const ooHandle(Assembly)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(Assembly)(const Assembly* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(Assembly)& operator=(const ooRef(Assembly)& id) { ooId::operator=(id); return *this; }
    ooRef(Assembly)& operator=(const ooShortRef(Assembly)& shortId);
    ooRef(Assembly)& operator=(const ooHandle(Assembly)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(Assembly)& operator=(const Assembly* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    Assembly* vm(const ooMode openMode = oocRead)
        { return (Assembly*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(Assembly) operator->() { return (ooDeref(Assembly)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(Assembly) : public ooShortRef(ooObj) {
  public:
    ooShortRef(Assembly)();
    ooShortRef(Assembly)(const ooShortRef(Assembly)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(Assembly)(const ooRef(Assembly)& id) { ooShortId::operator=(id); }
    ooShortRef(Assembly)(const ooHandle(Assembly)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(Assembly)(const Assembly* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(Assembly)& operator=(const ooShortRef(Assembly)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(Assembly)& operator=(const ooRef(Assembly)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(Assembly)& operator=(const ooHandle(Assembly)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(Assembly)& operator=(const Assembly* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(Assembly)::ooHandle(Assembly)(const ooRef(Assembly)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(Assembly)& ooHandle(Assembly)::operator=(const ooRef(Assembly)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(Assembly)& ooHandle(Assembly)::operator=(const ooShortRef(Assembly)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(Assembly)& ooRef(Assembly)::operator=(const ooShortRef(Assembly)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(ComplexAssembly);
extern const ooAssocNumber ooAssocN(ComplexAssembly,subAssemblies);


class ComplexAssembly;
class ooRef(ComplexAssembly);
class ooShortRef(ComplexAssembly);
class ooHandle(ComplexAssembly) : public ooHandle(Assembly) {
  public:
    ooHandle(ComplexAssembly)();
    ooHandle(ComplexAssembly)(const ooHandle(ComplexAssembly)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(ComplexAssembly)(const ooRef(ComplexAssembly)& id);
    ooHandle(ComplexAssembly)(const ComplexAssembly* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(ComplexAssembly)& operator=(const ooHandle(ComplexAssembly)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(ComplexAssembly)& operator=(const ooRef(ComplexAssembly)& id);
    ooHandle(ComplexAssembly)& operator=(const ooShortRef(ComplexAssembly)& shortId);
    ooHandle(ComplexAssembly)& operator=(const ComplexAssembly* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(ComplexAssembly), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(ComplexAssembly), openMode); }
    ooHandle(ComplexAssembly)& copy(const ooHandle(ooObj)& nearH, ooHandle(ComplexAssembly)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(ComplexAssembly) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(ComplexAssembly) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    ComplexAssembly* vm()         { return (ComplexAssembly*)get_vm(); }
    ComplexAssembly* operator->() { return (ComplexAssembly*)get_vm(); }
    ComplexAssembly& operator*();
    operator ComplexAssembly*();
};

class ooItr(ComplexAssembly) : public ooHandle(ComplexAssembly) {
  public:
    ooItr(ComplexAssembly)();
    ~ooItr(ComplexAssembly)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(ComplexAssembly)(ooItr(ComplexAssembly)&) { }
    void operator=(ooItr(ComplexAssembly)&) { }
};

class ooDeref(ComplexAssembly) : public ooDerefBase {
    friend ooRef(ComplexAssembly);
    ooDeref(ComplexAssembly)(ooDeref(ComplexAssembly)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(ComplexAssembly)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(ComplexAssembly)() { unpin(); }
public:
    ComplexAssembly *operator->() { return (ComplexAssembly *)vm(); }
};

class ooRef(ComplexAssembly) : public ooRef(Assembly) {
  public:
    ooRef(ComplexAssembly)() { }
    ooRef(ComplexAssembly)(const ooRef(ComplexAssembly)& id) : ooRef(Assembly)(id) { }
    ooRef(ComplexAssembly)(const ooHandle(ComplexAssembly)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(ComplexAssembly)(const ComplexAssembly* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(ComplexAssembly)& operator=(const ooRef(ComplexAssembly)& id) { ooId::operator=(id); return *this; }
    ooRef(ComplexAssembly)& operator=(const ooShortRef(ComplexAssembly)& shortId);
    ooRef(ComplexAssembly)& operator=(const ooHandle(ComplexAssembly)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(ComplexAssembly)& operator=(const ComplexAssembly* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    ComplexAssembly* vm(const ooMode openMode = oocRead)
        { return (ComplexAssembly*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(ComplexAssembly) operator->() { return (ooDeref(ComplexAssembly)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(ComplexAssembly) : public ooShortRef(Assembly) {
  public:
    ooShortRef(ComplexAssembly)();
    ooShortRef(ComplexAssembly)(const ooShortRef(ComplexAssembly)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(ComplexAssembly)(const ooRef(ComplexAssembly)& id) { ooShortId::operator=(id); }
    ooShortRef(ComplexAssembly)(const ooHandle(ComplexAssembly)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(ComplexAssembly)(const ComplexAssembly* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(ComplexAssembly)& operator=(const ooShortRef(ComplexAssembly)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(ComplexAssembly)& operator=(const ooRef(ComplexAssembly)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(ComplexAssembly)& operator=(const ooHandle(ComplexAssembly)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(ComplexAssembly)& operator=(const ComplexAssembly* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(ComplexAssembly)::ooHandle(ComplexAssembly)(const ooRef(ComplexAssembly)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(ComplexAssembly)& ooHandle(ComplexAssembly)::operator=(const ooRef(ComplexAssembly)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(ComplexAssembly)& ooHandle(ComplexAssembly)::operator=(const ooShortRef(ComplexAssembly)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(ComplexAssembly)& ooRef(ComplexAssembly)::operator=(const ooShortRef(ComplexAssembly)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(BaseAssembly);
extern const ooAssocNumber ooAssocN(BaseAssembly,componentsPriv);
extern const ooAssocNumber ooAssocN(BaseAssembly,componentsShar);


class BaseAssembly;
class ooRef(BaseAssembly);
class ooShortRef(BaseAssembly);
class ooHandle(BaseAssembly) : public ooHandle(Assembly) {
  public:
    ooHandle(BaseAssembly)();
    ooHandle(BaseAssembly)(const ooHandle(BaseAssembly)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(BaseAssembly)(const ooRef(BaseAssembly)& id);
    ooHandle(BaseAssembly)(const BaseAssembly* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(BaseAssembly)& operator=(const ooHandle(BaseAssembly)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(BaseAssembly)& operator=(const ooRef(BaseAssembly)& id);
    ooHandle(BaseAssembly)& operator=(const ooShortRef(BaseAssembly)& shortId);
    ooHandle(BaseAssembly)& operator=(const BaseAssembly* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(BaseAssembly), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(BaseAssembly), openMode); }
    ooHandle(BaseAssembly)& copy(const ooHandle(ooObj)& nearH, ooHandle(BaseAssembly)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(BaseAssembly) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(BaseAssembly) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    BaseAssembly* vm()         { return (BaseAssembly*)get_vm(); }
    BaseAssembly* operator->() { return (BaseAssembly*)get_vm(); }
    BaseAssembly& operator*();
    operator BaseAssembly*();
};

class ooItr(BaseAssembly) : public ooHandle(BaseAssembly) {
  public:
    ooItr(BaseAssembly)();
    ~ooItr(BaseAssembly)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(BaseAssembly)(ooItr(BaseAssembly)&) { }
    void operator=(ooItr(BaseAssembly)&) { }
};

class ooDeref(BaseAssembly) : public ooDerefBase {
    friend ooRef(BaseAssembly);
    ooDeref(BaseAssembly)(ooDeref(BaseAssembly)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(BaseAssembly)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(BaseAssembly)() { unpin(); }
public:
    BaseAssembly *operator->() { return (BaseAssembly *)vm(); }
};

class ooRef(BaseAssembly) : public ooRef(Assembly) {
  public:
    ooRef(BaseAssembly)() { }
    ooRef(BaseAssembly)(const ooRef(BaseAssembly)& id) : ooRef(Assembly)(id) { }
    ooRef(BaseAssembly)(const ooHandle(BaseAssembly)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(BaseAssembly)(const BaseAssembly* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(BaseAssembly)& operator=(const ooRef(BaseAssembly)& id) { ooId::operator=(id); return *this; }
    ooRef(BaseAssembly)& operator=(const ooShortRef(BaseAssembly)& shortId);
    ooRef(BaseAssembly)& operator=(const ooHandle(BaseAssembly)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(BaseAssembly)& operator=(const BaseAssembly* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    BaseAssembly* vm(const ooMode openMode = oocRead)
        { return (BaseAssembly*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(BaseAssembly) operator->() { return (ooDeref(BaseAssembly)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(BaseAssembly) : public ooShortRef(Assembly) {
  public:
    ooShortRef(BaseAssembly)();
    ooShortRef(BaseAssembly)(const ooShortRef(BaseAssembly)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(BaseAssembly)(const ooRef(BaseAssembly)& id) { ooShortId::operator=(id); }
    ooShortRef(BaseAssembly)(const ooHandle(BaseAssembly)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(BaseAssembly)(const BaseAssembly* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(BaseAssembly)& operator=(const ooShortRef(BaseAssembly)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(BaseAssembly)& operator=(const ooRef(BaseAssembly)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(BaseAssembly)& operator=(const ooHandle(BaseAssembly)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(BaseAssembly)& operator=(const BaseAssembly* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(BaseAssembly)::ooHandle(BaseAssembly)(const ooRef(BaseAssembly)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(BaseAssembly)& ooHandle(BaseAssembly)::operator=(const ooRef(BaseAssembly)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(BaseAssembly)& ooHandle(BaseAssembly)::operator=(const ooShortRef(BaseAssembly)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(BaseAssembly)& ooRef(BaseAssembly)::operator=(const ooShortRef(BaseAssembly)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(Module);
extern const ooAssocNumber ooAssocN(Module,designRoot);
extern const ooAssocNumber ooAssocN(Module,man);


class Module;
class ooRef(Module);
class ooShortRef(Module);
class ooHandle(Module) : public ooHandle(ooContObj) {
  public:
    ooHandle(Module)();
    ooHandle(Module)(const ooHandle(Module)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(Module)(const ooRef(Module)& id);
    ooHandle(Module)(const Module* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(Module)& operator=(const ooHandle(Module)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(Module)& operator=(const ooRef(Module)& id);
    ooHandle(Module)& operator=(const ooShortRef(Module)& shortId);
    ooHandle(Module)& operator=(const Module* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(Module), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(Module), openMode); }
    ooHandle(Module)& copy(const ooHandle(ooObj)& nearH, ooHandle(Module)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(Module) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(Module) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    Module* vm()         { return (Module*)get_vm(); }
    Module* operator->() { return (Module*)get_vm(); }
    Module& operator*();
    operator Module*();
};

class ooItr(Module) : public ooHandle(Module) {
  public:
    ooItr(Module)();
    ~ooItr(Module)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(Module)(ooItr(Module)&) { }
    void operator=(ooItr(Module)&) { }
};

class ooDeref(Module) : public ooDerefBase {
    friend ooRef(Module);
    ooDeref(Module)(ooDeref(Module)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(Module)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(Module)() { unpin(); }
public:
    Module *operator->() { return (Module *)vm(); }
};

class ooRef(Module) : public ooRef(ooContObj) {
  public:
    ooRef(Module)() { }
    ooRef(Module)(const ooRef(Module)& id) : ooRef(ooContObj)(id) { }
    ooRef(Module)(const ooHandle(Module)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(Module)(const Module* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(Module)& operator=(const ooRef(Module)& id) { ooId::operator=(id); return *this; }
    ooRef(Module)& operator=(const ooShortRef(Module)& shortId);
    ooRef(Module)& operator=(const ooHandle(Module)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(Module)& operator=(const Module* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    Module* vm(const ooMode openMode = oocRead)
        { return (Module*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(Module) operator->() { return (ooDeref(Module)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(Module) : public ooShortRef(ooContObj) {
  public:
    ooShortRef(Module)();
    ooShortRef(Module)(const ooShortRef(Module)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(Module)(const ooRef(Module)& id) { ooShortId::operator=(id); }
    ooShortRef(Module)(const ooHandle(Module)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(Module)(const Module* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(Module)& operator=(const ooShortRef(Module)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(Module)& operator=(const ooRef(Module)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(Module)& operator=(const ooHandle(Module)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(Module)& operator=(const Module* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(Module)::ooHandle(Module)(const ooRef(Module)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(Module)& ooHandle(Module)::operator=(const ooRef(Module)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(Module)& ooHandle(Module)::operator=(const ooShortRef(Module)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(Module)& ooRef(Module)::operator=(const ooShortRef(Module)& shortId)
    { ooId::operator=(shortId); return *this; }



extern const ooTypeNumber ooTypeN(DummyObj);


class DummyObj;
class ooRef(DummyObj);
class ooShortRef(DummyObj);
class ooHandle(DummyObj) : public ooHandle(ooObj) {
  public:
    ooHandle(DummyObj)();
    ooHandle(DummyObj)(const ooHandle(DummyObj)& objH) { opiCtHandleByHandle(*this, objH); }
    ooHandle(DummyObj)(const ooRef(DummyObj)& id);
    ooHandle(DummyObj)(const DummyObj* pObject) { opiCtHandleByPtr(*this, pObject); }
    ooHandle(DummyObj)& operator=(const ooHandle(DummyObj)& objH) { opiAsHandleByHandle(*this, objH); return *this; }
    ooHandle(DummyObj)& operator=(const ooRef(DummyObj)& id);
    ooHandle(DummyObj)& operator=(const ooShortRef(DummyObj)& shortId);
    ooHandle(DummyObj)& operator=(const DummyObj* pObject)
	{ (pObject) ? opiAsHandleByPtr(*this, pObject) : nullify(); return *this; }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, ooKey& key, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, key, ooTypeN(DummyObj), openMode); }
    ooStatus lookupObj(const ooHandle(ooObj)& scopeH, const char* name, const ooMode openMode = oocRead)
	{ return opiLookupObj(*this, scopeH, name, ooTypeN(DummyObj), openMode); }
    ooHandle(DummyObj)& copy(const ooHandle(ooObj)& nearH, ooHandle(DummyObj)& newObjH)
	{ opiCopyObj(*this, nearH, newObjH); return newObjH; }
    ooHandle(DummyObj) copy(const ooHandle(ooObj)& nearH)
	{ooHandle(DummyObj) newObjH; opiCopyObj(*this, nearH, newObjH); return newObjH;}
    DummyObj* vm()         { return (DummyObj*)get_vm(); }
    DummyObj* operator->() { return (DummyObj*)get_vm(); }
    DummyObj& operator*();
    operator DummyObj*();
};

class ooItr(DummyObj) : public ooHandle(DummyObj) {
  public:
    ooItr(DummyObj)();
    ~ooItr(DummyObj)();    
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode = oocNoOpen)
	{ return opiInitScanItr(*this, objH, mode); }
    ooStatus scan(const ooHandle(ooObj)& objH, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooObj)& objH, const ooMode mode, const ooAccessMode access, const char* predicate, ...);
    ooStatus scan(const ooHandle(ooContObj)& objH, const ooLookupKey& lookupKey, const ooMode mode = oocNoOpen)
	{ return opiInitScanIndexItr(*this, objH, lookupKey, mode); }
    ooTypeNumber typeN()   { return opiGetItrTypeN(*this); }
    ooStatus     end()     { return opiCloseItr(*this); }
    ooBoolean    next()    { return _itr.next(self()); }
    ooIterator&  get_itr() { return _itr; }
  private:
    ooIterator   _itr;
    ooItr(DummyObj)(ooItr(DummyObj)&) { }
    void operator=(ooItr(DummyObj)&) { }
};

class ooDeref(DummyObj) : public ooDerefBase {
    friend ooRef(DummyObj);
    ooDeref(DummyObj)(ooDeref(DummyObj)& deref) { set_vm(deref.vm()); pin(); }
    ooDeref(DummyObj)(ooObj *pObject) { set_vm(pObject);}
    ~ooDeref(DummyObj)() { unpin(); }
public:
    DummyObj *operator->() { return (DummyObj *)vm(); }
};

class ooRef(DummyObj) : public ooRef(ooObj) {
  public:
    ooRef(DummyObj)() { }
    ooRef(DummyObj)(const ooRef(DummyObj)& id) : ooRef(ooObj)(id) { }
    ooRef(DummyObj)(const ooHandle(DummyObj)& objH) { opiCtIdByHandle(*this, objH); }
    ooRef(DummyObj)(const DummyObj* pObject) { opiCtIdByPtr(*this, pObject); }
    ooRef(DummyObj)& operator=(const ooRef(DummyObj)& id) { ooId::operator=(id); return *this; }
    ooRef(DummyObj)& operator=(const ooShortRef(DummyObj)& shortId);
    ooRef(DummyObj)& operator=(const ooHandle(DummyObj)& objH) { opiCtIdByHandle(*this, objH); return *this; }
    ooRef(DummyObj)& operator=(const DummyObj* pObject) { opiAsIdByPtr(*this, pObject); return *this; }
    DummyObj* vm(const ooMode openMode = oocRead)
        { return (DummyObj*) opiOidVM(this, openMode); }
#ifndef __DECCXX
    ooDeref(DummyObj) operator->() { return (ooDeref(DummyObj)) opiOidVM(this, oocRead); }
#endif
};

class ooShortRef(DummyObj) : public ooShortRef(ooObj) {
  public:
    ooShortRef(DummyObj)();
    ooShortRef(DummyObj)(const ooShortRef(DummyObj)& shortId) { ooShortId::operator=(shortId); }
    ooShortRef(DummyObj)(const ooRef(DummyObj)& id) { ooShortId::operator=(id); }
    ooShortRef(DummyObj)(const ooHandle(DummyObj)& objH) { opiCtShortIdByHandle(*this, objH); }
    ooShortRef(DummyObj)(const DummyObj* pObject) { opiCtShortIdByPtr(*this, pObject); }
    ooShortRef(DummyObj)& operator=(const ooShortRef(DummyObj)& shortId) { ooShortId::operator=(shortId); return *this; }
    ooShortRef(DummyObj)& operator=(const ooRef(DummyObj)& id) { ooShortId::operator=(id); return *this; }
    ooShortRef(DummyObj)& operator=(const ooHandle(DummyObj)& objH) { opiCtShortIdByHandle(*this, objH); return *this; }
    ooShortRef(DummyObj)& operator=(const DummyObj* pObject) { opiAsShortIdByPtr(*this, pObject); return *this; }
};

inline ooHandle(DummyObj)::ooHandle(DummyObj)(const ooRef(DummyObj)& id)
    { opiCtHandleById(*this, id); }
inline ooHandle(DummyObj)& ooHandle(DummyObj)::operator=(const ooRef(DummyObj)& id)
    { opiAsHandleById(*this, id); return *this; }
inline ooHandle(DummyObj)& ooHandle(DummyObj)::operator=(const ooShortRef(DummyObj)& shortId)
    { opiAsHandleByShortId(*this, shortId); return *this; }
inline ooRef(DummyObj)& ooRef(DummyObj)::operator=(const ooShortRef(DummyObj)& shortId)
    { ooId::operator=(shortId); return *this; }


#endif /* _OO7_REF_H */

