/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <string.h>
#include <stdio.h>

#include <oo.h>
#include "OO7.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #1 - randomly choose "Query1RepeatCnt" atomic parts by 
// lookup on their id field.  An index can be used for the
// actual lookup.  
//
/////////////////////////////////////////////////////////////////


extern ooTrans	xact;
extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #1
//
//////////////////////////////////////////////////////////////////


int query1(ooHandle(ooDBObj)& databaseH)
{

    // get access to the design library containing atomic parts

    ooHandle(ooContObj) designLibH;
    designLibH.open(databaseH, "DesignLibrary", oocRead);
    if (designLibH.isValid() == oocFalse) {
        fprintf(stderr, "ERROR: Unable to access DesignLibrary\n");
	xact.abort();
        exit(1);
    }


    // prepare to select random atomic parts via their id index

    ooKey key;
    int partId;
    key.type = oocInt32;
    key.offset = ooGetMemberOffset(AtomicPart, id);
    key.size   = ooGetMemberSize(AtomicPart, id);
    key.value  = &partId;
    ooHandle(AtomicPart) atomH;

    // set random seed so "hot" runs are truly hot
    srandom(1);

    // now randomly select parts and process them

    for (int i = 0; i < Query1RepeatCnt; i++) 
    {
	// generate part id and lookup part
	partId = (int) (random() % TotalAtomicParts) + 1;
	atomH.lookupObj(designLibH, key);
        if (atomH.isValid() == oocFalse) {
            fprintf(stderr, "ERROR: Unable to find atomic part %d\n", partId);
   	    xact.abort();
            exit(1);
        }

        if (debugMode) {
            printf("    In Query1, partId = %d:\n", partId);
        }

	// process part by calling the null procedure
	atomH->DoNothing();
    }
    return Query1RepeatCnt;
}

